import logging

import tensorflow as tf

from .modeling_albert import (Albert, AlbertEmbedding, AlbertEncoder,
                              AlbertEncoderGroup, AlbertEncoderLayer)
from .modeling_bert import (Bert, BertEmbedding, BertEncoder, BertEncoderLayer,
                            BertIntermediate, BertPretrainedModel)
from .modeling_utils import complete_inputs

__name__ = 'transformers_keras'
__version__ = '0.3.1'

logging.basicConfig(format="%(asctime)s %(levelname)7s %(filename)15s %(lineno)4d] %(message)s", level=logging.INFO)



import logging
import os

import numpy as np
import tensorflow as tf


def gelu(x):
    """ Gaussian Error Linear Unit.
    Original Implementation of the gelu activation function in Google Bert repo when initially created.
        For information: OpenAI GPT's gelu is slightly different (and gives slightly different results):
        0.5 * x * (1 + torch.tanh(math.sqrt(2 / math.pi) * (x + 0.044715 * torch.pow(x, 3))))
        Also see https://arxiv.org/abs/1606.08415
    """
    cdf = 0.5 * (1.0 + tf.math.erf(x / tf.math.sqrt(2.0)))
    return x * cdf


def gelu_new(x):
    """Gaussian Error Linear Unit.
    This is a smoother version of the RELU.
    Original paper: https://arxiv.org/abs/1606.08415
    Args:
        x: float Tensor to perform activation.
    Returns:
        `x` with the GELU activation applied.
    """
    cdf = 0.5 * (1.0 + tf.tanh((np.sqrt(2 / np.pi) * (x + 0.044715 * tf.pow(x, 3)))))
    return x * cdf


def swish(x):
    return x * tf.sigmoid(x)


ACT2FN = {
    "gelu": tf.keras.layers.Activation(gelu),
    "relu": tf.keras.activations.relu,
    "swish": tf.keras.layers.Activation(swish),
    "gelu_new": tf.keras.layers.Activation(gelu_new),
}


def choose_activation(act):
    if isinstance(act, tf.keras.layers.Activation):
        return act
    if isinstance(act, str):
        return ACT2FN[act]
    return act


def initialize(stddev=0.02):
    return tf.keras.initializers.TruncatedNormal(stddev=stddev)


def parse_pretrained_model_files(pretrain_model_dir):
    config_file, ckpt, vocab = None, None, None
    pretrain_model_dir = os.path.abspath(pretrain_model_dir)
    if not os.path.exists(pretrain_model_dir):
        logging.info('pretrain model dir: {} is not exists.'.format(pretrain_model_dir))
        return config_file, ckpt, vocab
    for f in os.listdir(pretrain_model_dir):
        if str(f).endswith('config.json'):
            config_file = os.path.join(pretrain_model_dir, f)
        if 'vocab' in str(f):
            vocab = os.path.join(pretrain_model_dir, f)
        if 'ckpt' in str(f):
            n = '.'.join(str(f).split('.')[:-1])
            ckpt = os.path.join(pretrain_model_dir, n)
    return config_file, ckpt, vocab


def unpack_inputs_2(inputs):
    if not isinstance(inputs, (list, tuple)):
        raise ValueError('Invalid inputs type! Inputs type must be a list or tuple!')
    inputs = list(inputs)
    if len(inputs) == 0:
        raise ValueError('Invalid inputs, must be not empty!')
    if len(inputs) == 1:
        input_ids, segment_ids = inputs[0], None
    if len(inputs) == 2:
        input_ids, segment_ids = inputs[0], inputs[1]
    if segment_ids is None:
        segment_ids = tf.cast(tf.fill(tf.shape(input_ids), 0), dtype=tf.int32)
    return input_ids, segment_ids


def unpack_inputs_3(inputs):
    if not isinstance(inputs, (list, tuple)):
        raise ValueError('Invalid inputs type! Inputs type must be a list or tuple!')
    inputs = list(inputs)
    if len(inputs) == 0:
        raise ValueError('Invalid inputs, must be not empty!')
    if len(inputs) == 1:
        input_ids, segment_ids, mask = inputs[0], None, None
    if len(inputs) == 2:
        input_ids, segment_ids, mask = inputs[0], inputs[1], None
    if len(inputs) == 3:
        input_ids, segment_ids, mask = inputs[0], inputs[1], inputs[2]
    if segment_ids is None:
        segment_ids = tf.cast(tf.fill(tf.shape(input_ids), 0), dtype=tf.int32)
    if mask is None:
        mask = tf.cast(tf.greater(input_ids, 0), dtype=tf.int32)
    return input_ids, segment_ids, mask


def complete_inputs(input_ids, segment_ids, attention_mask):
    assert input_ids is not None
    if segment_ids is None:
        segment_ids = tf.cast(tf.fill(tf.shape(input_ids), 0), dtype=tf.int32)
    if attention_mask is None:
        attention_mask = tf.cast(tf.greater(input_ids, 0), dtype=tf.int32)
    return input_ids, segment_ids, attention_mask



import json
import os

import tensorflow as tf
from tokenizers import BertWordPieceTokenizer
from transformers_keras.modeling_albert import Albert
from transformers_keras.modeling_bert import Bert

BASE_DIR = os.environ.get('PRETRAINED_MODE_PATH', None)


class LoadPretrainedModelTest(tf.test.TestCase):

    def _build_bert_inputs(self):
        vocab_path = os.path.join(BASE_DIR, 'bert_uncased_L-6_H-768_A-12', 'vocab.txt')
        tokenizer = BertWordPieceTokenizer(vocab_path)
        encoding = tokenizer.encode('我爱NLP')
        input_ids = tf.constant([encoding.ids], dtype=tf.int32, shape=(1, len(encoding.ids)))
        segment_ids = tf.constant([encoding.type_ids], dtype=tf.int32, shape=(1, len(encoding.type_ids)))
        attention_mask = tf.constant([encoding.attention_mask], dtype=tf.int32, shape=(1, len(encoding.attention_mask)))
        return input_ids, segment_ids, attention_mask

    def _build_model_from_transformers(self):
        from transformers import TFBertModel
        model = TFBertModel.from_pretrained(
            os.path.join(BASE_DIR, 'bert_uncased_L-6_H-768_A-12-pytorch'), from_pt=True)
        return model

    def _build_model(self):
        model = Bert.from_pretrained(
            os.path.join(BASE_DIR, 'bert_uncased_L-6_H-768_A-12'),
            return_states=True, verbose=False)
        return model

    def test_bert_comprare_with_transformers(self):
        model = self._build_model()
        transformer_model = self._build_model_from_transformers()

        input_ids, segment_ids, attention_mask = self._build_bert_inputs()

        def _comprare_embedding_output():
            a = model.bert_embedding(input_ids, segment_ids)
            embedding = transformer_model.get_layer('bert').embeddings
            b = embedding(input_ids=input_ids, token_type_ids=segment_ids, position_ids=None, training=False)
            self.assertAllClose(a, b)

        _comprare_embedding_output()

        def _compare_final_outputs():
            a_sequence_output, a_pooled_output, _ = model(inputs=[input_ids, segment_ids, attention_mask])
            b_sequence_output, b_pooled_output = transformer_model(
                input_ids=input_ids, token_type_ids=segment_ids, attention_mask=attention_mask, return_dict=False)
            # b_sequence_output, b_pooled_output = b_outputs[0], b_outputs[1]
            self.assertAllClose(a_sequence_output, b_sequence_output, rtol=0.5, atol=1e-4)
            self.assertAllClose(a_pooled_output, b_pooled_output, rtol=0.02, atol=1e-4)

        _compare_final_outputs()

    def _build_albert_from_transformers(self):
        from transformers import TFAlbertModel
        model = TFAlbertModel.from_pretrained(os.path.join(BASE_DIR, 'albert_base_zh_pytorch'), from_pt=True)
        return model

    def test_albert_compare_with_transformers(self):
        model = Albert.from_pretrained(os.path.join(BASE_DIR, 'albert_base_zh'))
        transformer_model = self._build_albert_from_transformers()

        input_ids, segment_ids, attention_mask = self._build_bert_inputs()

        def _compare_embedding_output():
            a_output = model.embedding(input_ids, segment_ids)
            b_output = transformer_model.get_layer('albert').embeddings(
                input_ids=input_ids, token_type_ids=segment_ids, position_ids=None, training=False)
            self.assertAllClose(a_output, b_output)

        def _compare_mapping_in_output():
            a_layer = model.encoder.embedding_mapping
            b_layer = transformer_model.get_layer('albert').encoder.embedding_hidden_mapping_in
            a_output = a_layer(model.embedding(input_ids, segment_ids))
            b_output = b_layer(transformer_model.get_layer('albert').embeddings(
                input_ids=input_ids, token_type_ids=segment_ids, position_ids=None, training=False))
            self.assertAllClose(a_output, b_output)

        def _compare_final_outputs():
            a_sequence_output, a_pooled_output = model(input_ids, segment_ids)
            b_sequence_output, b_pooled_output = transformer_model(
                input_ids=input_ids, token_type_ids=segment_ids, attention_mask=attention_mask, return_dict=False)
            self.assertAllClose(a_sequence_output, b_sequence_output, rtol=0.05, atol=1e-5)
            self.assertAllClose(a_pooled_output, b_pooled_output, rtol=0.002, atol=1e-5)

        _compare_embedding_output()
        _compare_mapping_in_output()
        _compare_final_outputs()

    def _do_predict(self, model):
        input_ids = tf.constant([1, 2, 3, 4, 5, 6, 7, 8], shape=(2, 4))
        # output_1 should be all close to output_2
        _, outputs_1 = model(inputs=[input_ids], training=False)
        print(outputs_1)
        # _, outputs_2 = model(input_ids, None, None)
        # print(outputs_2)

    def test_load_pretrained_bert(self):
        model_paths = [
            'chinese_wwm_ext_L-12_H-768_A-12',
            'chinese_L-12_H-768_A-12',
            'chinese_roberta_wwm_ext_L-12_H-768_A-12',
            'chinese_roberta_wwm_large_ext_L-24_H-1024_A-16'
        ]
        for p in model_paths:
            model = Bert.from_pretrained(os.path.join(BASE_DIR, p), verbose=True)
            model.summary()
            self._do_predict(model)

        # skip weights
        model = Bert.from_pretrained(
            os.path.join(BASE_DIR, model_paths[0]),
            skip_token_embedding=True,
            skip_pooler=True,
            verbose=False)
        model.summary()
        self._do_predict(model)

    def test_load_pretrained_albert(self):
        model_paths = [
            'albert_base_zh', 'albert_large_zh', 'albert_xlarge_zh'
        ]
        for p in model_paths:
            model = Albert.from_pretrained(os.path.join(BASE_DIR, p))
            model.summary()
            self._do_predict(model)

        model = Albert.from_pretrained(
            os.path.join(BASE_DIR, model_paths[0]),
            skip_token_embedding=True,
            skip_pooler=True,)
        model.summary()
        self._do_predict(model)

    def test_bert_classify(self):

        def _build_bert_model(trainable=True):
            input_ids = tf.keras.layers.Input(shape=(None,), dtype=tf.int32, name='input_ids')
            segment_ids = tf.keras.layers.Input(shape=(None,), dtype=tf.int32, name='segment_ids')

            bert = Bert.from_pretrained(os.path.join(BASE_DIR, 'chinese_roberta_wwm_ext_L-12_H-768_A-12'))
            bert.trainable = trainable

            sequence_output, pooled_output = bert(inputs=[input_ids, segment_ids])
            outputs = tf.keras.layers.Dense(2, name='output')(pooled_output)
            model = tf.keras.Model(inputs=[input_ids, segment_ids], outputs=outputs)
            model.compile(loss='binary_cross_entropy', optimizer='adam')
            return model

        for trainable in [True]:
            model = _build_bert_model(trainable)
            model.summary()

            model.save('models/export-bert-classify/1', include_optimizer=False)

    def test_albert_classify(self):

        def _build_albert_model(trainable=True):
            input_ids = tf.keras.layers.Input(shape=(None,), dtype=tf.int32, name='input_ids')
            segment_ids = tf.keras.layers.Input(shape=(None,), dtype=tf.int32, name='segment_ids')

            albert = Albert.from_pretrained(os.path.join(BASE_DIR, 'albert_base_zh'))
            albert.trainable = trainable

            sequence_output, pooled_output = albert(input_ids, segment_ids, None)
            outputs = tf.keras.layers.Dense(2, name='output')(pooled_output)
            model = tf.keras.Model(inputs=[input_ids, segment_ids], outputs=outputs)
            model.compile(loss='binary_cross_entropy', optimizer='adam')
            return model

        for trainable in [True, False]:
            model = _build_albert_model(trainable)
            model.summary()


if __name__ == "__main__":
    tf.test.main()



import json
import logging
import os

import numpy as np
import tensorflow as tf

from .abstract_adapter import AbstractAdapter, zip_weights


class BertAdapter(AbstractAdapter):

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    def adapte_config(self, config_file, **kwrgs):
        with open(config_file, mode='rt', encoding='utf8') as fin:
            config = json.load(fin)

        model_config = {
            'vocab_size': config['vocab_size'],
            'activation': config['hidden_act'],
            'max_positions': config['max_position_embeddings'],
            'hidden_size': config['hidden_size'],
            'type_vocab_size': config['type_vocab_size'],
            'intermediate_size': config['intermediate_size'],
            'hidden_dropout_rate': config['hidden_dropout_prob'],
            'attention_dropout_rate': config['attention_probs_dropout_prob'],
            'initializer_range': config['initializer_range'],
            'num_layers': config['num_hidden_layers'],
            'num_attention_heads': config['num_attention_heads'],
        }
        return model_config

    def adapte_weights(self, bert, config, ckpt, **kwargs):
        logging.info('Bert model name: %s', bert.name)
        model_prefix = bert.name
        if kwargs.get('model_prefix', ''):
            model_prefix = kwargs['model_prefix'] + '/' + bert.name
        logging.info('Using model prefix: %s', model_prefix)
        mapping = {}
        ckpt_prefix = kwargs.get('ckpt_prefix', 'bert')
        ckpt_weight_names = [x[0] for x in tf.train.list_variables(ckpt)]
        self_weight_names = set([x.name for x in bert.trainable_weights])
        for w in ckpt_weight_names:
            if any(x in w for x in ['embeddings', 'pooler', 'encoder']):
                mw = model_prefix + w.lstrip(ckpt_prefix) + ':0'
                if mw not in self_weight_names:
                    logging.warning('weight: %s not in model weights', mw)
                    continue
                mapping[mw] = w

        if self.skip_token_embedding:
            self._skip_weight(mapping, model_prefix + '/embeddings/word_embeddings:0')
        if self.skip_position_embedding:
            self._skip_weight(mapping, model_prefix + '/embeddings/position_embeddings:0')
        if self.skip_segment_embedding:
            self._skip_weight(mapping, model_prefix + '/embeddings/token_type_embeddings:0')
        if self.skip_embedding_layernorm:
            self._skip_weight(mapping, model_prefix + '/embeddings/LayerNorm/gamma:0')
            self._skip_weight(mapping, model_prefix + '/embeddings/LayerNorm/beta:0')
        if self.skip_pooler:
            self._skip_weight(mapping, model_prefix + '/pooler/dense/kernel:0')
            self._skip_weight(mapping, model_prefix + '/pooler/dense/bias:0')

        # zip weight names and values
        zipped_weights = zip_weights(
            bert,
            ckpt,
            mapping,
            **kwargs)
        # set values to weights
        tf.keras.backend.batch_set_value(zipped_weights)

        if not kwargs.get('check_weights', False):
            return

        self_weights = {w.name: w.numpy() for w in bert.trainable_weights}
        for k, v in self_weights.items():
            ckpt_key = mapping.get(k, None)
            if not ckpt_key:
                continue
            ckpt_value = tf.train.load_variable(ckpt, ckpt_key)
            if ckpt_value is None:
                logging.warning('ckpt value is None of key: %s', ckpt_key)
            assert np.allclose(v, ckpt_value)

    def _skip_weight(self, mapping, name):
        mapping.pop(name)
        logging.info('Skip load weight: %s', name)



import json
import logging
import os

import tensorflow as tf

from transformers_keras.adapters import parse_pretrained_model_files
from transformers_keras.adapters.bert_adapter import BertAdapter

from .modeling_utils import choose_activation, unpack_inputs_3


class BertEmbedding(tf.keras.layers.Layer):

    def __init__(self,
                 vocab_size=21128,
                 max_positions=512,
                 embedding_size=768,
                 type_vocab_size=2,
                 hidden_dropout_rate=0.1,
                 initializer_range=0.02,
                 epsilon=1e-12,
                 **kwargs):
        super().__init__(**kwargs)
        assert vocab_size > 0, "vocab_size must greater than 0."
        self.vocab_size = vocab_size
        self.max_positions = max_positions
        self.type_vocab_size = type_vocab_size
        self.embedding_size = embedding_size
        self.initializer_range = initializer_range
        self.layernorm = tf.keras.layers.LayerNormalization(epsilon=epsilon, name='LayerNorm')
        self.dropout = tf.keras.layers.Dropout(hidden_dropout_rate)

    def build(self, input_shape):
        self.token_embedding = self.add_weight(
            'word_embeddings',
            shape=[self.vocab_size, self.embedding_size],
            initializer=tf.keras.initializers.TruncatedNormal(stddev=self.initializer_range),
        )
        self.position_embedding = self.add_weight(
            'position_embeddings',
            shape=[self.max_positions, self.embedding_size],
            initializer=tf.keras.initializers.TruncatedNormal(stddev=self.initializer_range),
        )
        self.token_type_embedding = self.add_weight(
            'token_type_embeddings',
            shape=[self.type_vocab_size, self.embedding_size],
            initializer=tf.keras.initializers.TruncatedNormal(stddev=self.initializer_range),
        )
        return super().build(input_shape)

    @property
    def embedding_table(self):
        return self.token_embedding

    def call(self, input_ids, segment_ids=None, position_ids=None, training=None):
        if segment_ids is None:
            segment_ids = tf.zeros_like(input_ids)
        if position_ids is None:
            position_ids = tf.range(0, tf.shape(input_ids)[1], dtype=input_ids.dtype)
            position_ids = tf.expand_dims(position_ids, axis=0)

        position_embeddings = tf.gather(self.position_embedding, position_ids)
        position_embeddings = tf.tile(position_embeddings, multiples=[tf.shape(input_ids)[0], 1, 1])
        token_type_embeddings = tf.gather(self.token_type_embedding, segment_ids)
        token_embeddings = tf.gather(self.token_embedding, input_ids)

        embeddings = token_embeddings + token_type_embeddings + position_embeddings
        embeddings = self.layernorm(embeddings)
        embeddings = self.dropout(embeddings, training=training)
        return embeddings


class BertMultiHeadAtttetion(tf.keras.layers.Layer):

    def __init__(self,
                 hidden_size=768,
                 num_attention_heads=8,
                 attention_dropout_rate=0.1,
                 initializer_range=0.02,
                 epsilon=1e-8,
                 **kwargs):
        super().__init__(**kwargs)
        self.num_attention_heads = num_attention_heads
        self.hidden_size = hidden_size
        self.query_weight = tf.keras.layers.Dense(
            self.hidden_size,
            kernel_initializer=tf.keras.initializers.TruncatedNormal(stddev=initializer_range),
            name='query')
        self.key_weight = tf.keras.layers.Dense(
            self.hidden_size,
            kernel_initializer=tf.keras.initializers.TruncatedNormal(stddev=initializer_range),
            name='key')
        self.value_weight = tf.keras.layers.Dense(
            self.hidden_size,
            kernel_initializer=tf.keras.initializers.TruncatedNormal(stddev=initializer_range),
            name='value')
        self.attention_dropout = tf.keras.layers.Dropout(attention_dropout_rate)

    def _split_heads(self, x, batch_size):
        x = tf.reshape(x, (batch_size, -1, self.num_attention_heads, self.hidden_size // self.num_attention_heads))
        return tf.transpose(x, perm=[0, 2, 1, 3])

    def _scaled_dot_product_attention(self, query, key, value, attention_mask, training=None):
        query = tf.cast(query, dtype=self.dtype)
        key = tf.cast(key, dtype=self.dtype)
        value = tf.cast(value, dtype=self.dtype)

        score = tf.matmul(query, key, transpose_b=True)
        dk = tf.cast(tf.shape(query)[-1], self.dtype)
        score = score / tf.math.sqrt(dk)
        if attention_mask is not None:
            attention_mask = tf.cast(attention_mask, dtype=self.dtype)
            score += tf.cast((1.0 - attention_mask) * -10000.0, dtype=self.dtype)
        attn_weights = tf.nn.softmax(score, axis=-1)
        attn_weights = self.attention_dropout(attn_weights, training=training)
        context = tf.matmul(attn_weights, value)
        return context, attn_weights

    def call(self, query, key, value, attention_mask, training=None):
        batch_size = tf.shape(query)[0]
        query = self._split_heads(self.query_weight(query), batch_size)
        key = self._split_heads(self.key_weight(key), batch_size)
        value = self._split_heads(self.value_weight(value), batch_size)
        context, attn_weights = self._scaled_dot_product_attention(
            query, key, value, attention_mask, training=training)
        context = tf.transpose(context, perm=[0, 2, 1, 3])
        context = tf.reshape(context, [batch_size, -1, self.hidden_size])
        return context, attn_weights


class BertAttentionOutput(tf.keras.layers.Layer):

    def __init__(self,
                 hidden_size=768,
                 hidden_dropout_rate=0.1,
                 initializer_range=0.02,
                 epsilon=1e-5,
                 **kwargs):
        super().__init__(**kwargs)
        self.dense = tf.keras.layers.Dense(
            hidden_size,
            kernel_initializer=tf.keras.initializers.TruncatedNormal(stddev=initializer_range),
            name='dense')
        self.dropout = tf.keras.layers.Dropout(hidden_dropout_rate)
        self.layernorm = tf.keras.layers.LayerNormalization(epsilon=epsilon, name='LayerNorm')

    def call(self, input_states, hidden_states, training=None):
        hidden_states = self.dense(hidden_states)
        hidden_states = self.dropout(hidden_states, training=training)
        hidden_states = self.layernorm(hidden_states + input_states)
        return hidden_states


class BertAttention(tf.keras.layers.Layer):

    def __init__(self,
                 hidden_size=768,
                 num_attention_heads=8,
                 hidden_dropout_rate=0.1,
                 attention_dropout_rate=0.1,
                 initializer_range=0.02,
                 epsilon=1e-5,
                 **kwargs):
        super().__init__(**kwargs)
        self.attention = BertMultiHeadAtttetion(
            hidden_size=hidden_size,
            num_attention_heads=num_attention_heads,
            attention_dropout_rate=attention_dropout_rate,
            initializer_range=initializer_range,
            epsilon=epsilon,
            name='self')
        self.attention_output = BertAttentionOutput(
            hidden_size=hidden_size,
            hidden_dropout_rate=hidden_dropout_rate,
            initializer_range=initializer_range,
            epsilon=epsilon,
            name='output')

    def call(self, hidden_states, attention_mask, training=None):
        context, attention_weights = self.attention(
            hidden_states, hidden_states, hidden_states, attention_mask, training=training)
        outputs = self.attention_output(hidden_states, context, training=training)
        return outputs, attention_weights


class BertIntermediate(tf.keras.layers.Layer):

    def __init__(self, intermediate_size=3072, activation='gelu', initializer_range=0.02, **kwargs):
        super().__init__(**kwargs)
        self.dense = tf.keras.layers.Dense(
            intermediate_size,
            kernel_initializer=tf.keras.initializers.TruncatedNormal(stddev=initializer_range),
            name='dense')
        self.activation = choose_activation(activation)

    def call(self, hidden_states):
        hidden_states = self.dense(hidden_states)
        hidden_states = self.activation(hidden_states)
        return hidden_states


class BertIntermediateOutput(tf.keras.layers.Layer):

    def __init__(self, hidden_size=768, hidden_dropout_rate=0.1, initializer_range=0.02, epsilon=1e-5, **kwargs):
        super().__init__(**kwargs)
        self.dense = tf.keras.layers.Dense(
            hidden_size,
            kernel_initializer=tf.keras.initializers.TruncatedNormal(stddev=initializer_range),
            name='dense')
        self.dropout = tf.keras.layers.Dropout(hidden_dropout_rate)
        self.layernorm = tf.keras.layers.LayerNormalization(epsilon=epsilon, name='LayerNorm')

    def call(self, input_states, hidden_states, training=None):
        hidden_states = self.dropout(self.dense(hidden_states), training=training)
        hidden_states = self.layernorm(hidden_states + input_states)
        return hidden_states


class BertEncoderLayer(tf.keras.layers.Layer):

    def __init__(self,
                 hidden_size=768,
                 num_attention_heads=8,
                 intermediate_size=3072,
                 activation='gelu',
                 hidden_dropout_rate=0.2,
                 attention_dropout_rate=0.1,
                 epsilon=1e-12,
                 initializer_range=0.02,
                 **kwargs):
        super().__init__(**kwargs)
        # attention block
        self.attention = BertAttention(
            hidden_size=hidden_size,
            num_attention_heads=num_attention_heads,
            hidden_dropout_rate=hidden_dropout_rate,
            attention_dropout_rate=attention_dropout_rate,
            initializer_range=initializer_range,
            name='attention')
        # intermediate block
        self.intermediate = BertIntermediate(
            intermediate_size=intermediate_size,
            activation=activation,
            initializer_range=initializer_range,
            name='intermediate')
        # output block
        self.intermediate_output = BertIntermediateOutput(
            hidden_size=hidden_size,
            hidden_dropout_rate=hidden_dropout_rate,
            initializer_range=initializer_range,
            epsilon=epsilon,
            name='output')

    def call(self, hidden_states, attn_mask, training=None):
        attn_output, attn_weights = self.attention(hidden_states, attn_mask, training=training)
        outputs = self.intermediate(attn_output)
        outputs = self.intermediate_output(attn_output, outputs, training=training)
        return outputs, attn_weights


class BertEncoder(tf.keras.layers.Layer):

    def __init__(self,
                 num_layers=6,
                 hidden_size=768,
                 num_attention_heads=8,
                 intermediate_size=3072,
                 activation='gelu',
                 hidden_dropout_rate=0.2,
                 attention_dropout_rate=0.1,
                 epsilon=1e-12,
                 initializer_range=0.02,
                 **kwargs):
        super().__init__(**kwargs)
        self.encoder_layers = [
            BertEncoderLayer(
                hidden_size=hidden_size,
                num_attention_heads=num_attention_heads,
                intermediate_size=intermediate_size,
                activation=activation,
                hidden_dropout_rate=hidden_dropout_rate,
                attention_dropout_rate=attention_dropout_rate,
                epsilon=epsilon,
                initializer_range=initializer_range,
                name='layer_{}'.format(i)
            ) for i in range(num_layers)
        ]

    def call(self, hidden_states, attention_mask, training=None):
        all_hidden_states = []
        all_attention_scores = []
        for _, encoder in enumerate(self.encoder_layers):
            hidden_states, attention_score = encoder(hidden_states, attention_mask, training=training)
            all_hidden_states.append(hidden_states)
            all_attention_scores.append(attention_score)
        # stack all_hidden_states to shape:
        # [batch_size, num_layers, num_attention_heads, hidden_size]
        all_hidden_states = tf.stack(all_hidden_states, axis=0)
        all_hidden_states = tf.transpose(all_hidden_states, perm=[1, 0, 2, 3])
        # stack all_attention_scores to shape:
        # [batch_size, num_layers, num_attention_heads, seqeucen_length, sequence_length]
        all_attention_scores = tf.stack(all_attention_scores, axis=0)
        all_attention_scores = tf.transpose(all_attention_scores, perm=[1, 0, 2, 3, 4])
        return hidden_states, all_hidden_states, all_attention_scores


class BertPooler(tf.keras.layers.Layer):

    def __init__(self, hidden_size=768, initializer_range=0.02, **kwargs):
        super().__init__(**kwargs)
        self.dense = tf.keras.layers.Dense(
            hidden_size,
            kernel_initializer=tf.keras.initializers.TruncatedNormal(stddev=initializer_range),
            activation='tanh',
            name='dense')

    def call(self, inputs):
        hidden_states = inputs
        # pool the first token: [CLS]
        outputs = self.dense(hidden_states[:, 0])
        return outputs


class BertPretrainedModel(tf.keras.Model):

    def __init__(self, return_states=False, return_attention_weights=False, **kwargs):
        unused_keys = []
        for k, v in kwargs.items():
            if str(k).startswith('skip_'):
                unused_keys.append(k)
        for k in unused_keys:
            kwargs.pop(k, None)
        super().__init__(**kwargs)
        self.return_states = return_states
        self.return_attention_weights = return_attention_weights

    @tf.function(input_signature=[
        {
            'input_ids': tf.TensorSpec(shape=(None, None), dtype=tf.int32, name='input_ids'),
            'segment_ids': tf.TensorSpec(shape=(None, None), dtype=tf.int32, name='segment_ids'),
            'attention_mask': tf.TensorSpec(shape=(None, None), dtype=tf.int32, name='attention_mask')
        }
    ])
    def serving(self, inputs):
        return self.forward(inputs)

    def forward(self, inputs):
        """Forward pass for serving.

        Arguments:
            inputs: A dict, contains `input_ids`, `segment_ids` and `attention_mask`

        Returns:
            outputs: A dict, contains outputs from serving
        """
        raise NotImplementedError()

    def dummy_inputs(self):
        input_ids = tf.constant([0] * 128, dtype=tf.int32, shape=(1, 128))
        segment_ids = tf.constant([0] * 128, dtype=tf.int32, shape=(1, 128))
        attn_mask = tf.constant([1] * 128, dtype=tf.int32, shape=(1, 128))
        return input_ids, segment_ids, attn_mask

    @classmethod
    def _build_extra_config(cls, model_config, **kwargs):
        extra_config = {}
        for k, v in kwargs.items():
            if k in ['verbose']:
                continue
            if k not in model_config:
                if str(k).startswith('override_'):
                    k = k.lstrip('override_')
                extra_config[k] = v
        if extra_config:
            logging.info('Load extra config: \n%s', json.dumps(extra_config, indent=4))
        return extra_config

    @classmethod
    def _merge_config(cls, model_config, extra_config):
        mixed_config = {}
        mixed_config.update(**model_config)
        mixed_config.update(**extra_config)
        logging.info('Using mixed config: \n%s', json.dumps(mixed_config, indent=4))
        return mixed_config

    @classmethod
    def from_pretrained(cls, pretrained_model_dir, adapter=None, **kwargs):
        config_file, ckpt, vocab_file = parse_pretrained_model_files(pretrained_model_dir)
        if not adapter:
            adapter = BertAdapter(**kwargs)
        model_config = adapter.adapte_config(config_file, **kwargs)
        logging.info('Load model config: \n%s', json.dumps(model_config, indent=4))
        extra_config = cls._build_extra_config(model_config, **kwargs)
        mixed_config = cls._merge_config(model_config, extra_config)
        model = cls(**mixed_config)
        input_ids, segment_ids, attn_mask = model.dummy_inputs()
        model(inputs=[input_ids, segment_ids, attn_mask], training=False)
        adapter.adapte_weights(
            model.bert_model,
            model_config,
            ckpt,
            model_prefix=model.name_prefix,
            **kwargs)
        return model

    @classmethod
    def from_config_file(cls, config_file, **kwargs):
        with open(config_file, mode='rt', encoding='utf-8') as fin:
            model_config = json.load(fin)
        logging.info('Load model config: \n%s', json.dumps(model_config, indent=4))
        extra_config = cls._build_extra_config(model_config, **kwargs)
        mixed_config = cls._merge_config(model_config, extra_config)
        model = cls(**mixed_config)
        input_ids, segment_ids, attn_mask = model.dummy_inputs()
        model(input_ids, segment_ids, attn_mask, training=False)
        return model

    @property
    def bert_model(self):
        return self

    @property
    def name_prefix(self):
        return ''


class Bert(BertPretrainedModel):

    def __init__(self,
                 vocab_size=21128,
                 max_positions=512,
                 hidden_size=768,
                 type_vocab_size=2,
                 num_layers=6,
                 num_attention_heads=8,
                 intermediate_size=3072,
                 activation='gelu',
                 hidden_dropout_rate=0.2,
                 attention_dropout_rate=0.1,
                 initializer_range=0.02,
                 epsilon=1e-12,
                 return_states=False,
                 return_attention_weights=False,
                 **kwargs):
        super().__init__(
            return_states=return_states,
            return_attention_weights=return_attention_weights,
            **kwargs)

        self.vocab_size = vocab_size
        self.type_vocab_size = type_vocab_size
        self.max_positions = max_positions
        self.num_layers = num_layers
        self.hidden_size = hidden_size
        self.num_attention_heads = num_attention_heads
        self.intermediate_size = intermediate_size
        self.hidden_dropout_rate = hidden_dropout_rate
        self.attention_dropout_rate = attention_dropout_rate
        self.initializer_range = initializer_range
        self.initialize_range = initializer_range

        self.bert_embedding = BertEmbedding(
            vocab_size=vocab_size,
            max_positions=max_positions,
            embedding_size=hidden_size,
            type_vocab_size=type_vocab_size,
            hidden_dropout_rate=hidden_dropout_rate,
            initializer_range=initializer_range,
            epsilon=epsilon,
            name='embeddings')

        self.bert_encoder = BertEncoder(
            num_layers=num_layers,
            hidden_size=hidden_size,
            num_attention_heads=num_attention_heads,
            intermediate_size=intermediate_size,
            activation=activation,
            hidden_dropout_rate=hidden_dropout_rate,
            attention_dropout_rate=attention_dropout_rate,
            epsilon=epsilon,
            initializer_range=initializer_range,
            name='encoder')

        self.bert_pooler = BertPooler(hidden_size=hidden_size, initializer_range=initializer_range, name='pooler')

    def get_embedding_table(self):
        return self.bert_embedding.embedding_table

    def forward(self, inputs):
        input_ids, segment_ids, attention_mask = inputs['input_ids'], inputs['segment_ids'], inputs['attention_mask']
        outputs = self(inputs=[input_ids, segment_ids, attention_mask])
        outputs = list(outputs)
        results = {
            'sequence_output': outputs.pop(0),
            'pooled_output': outputs.pop(0),
        }
        if self.return_states:
            results['hidden_states'] = outputs.pop(0)
        if self.return_attention_weights:
            results['attention_weights'] = outputs.pop(0)
        return results

    def call(self, inputs, training=None):
        input_ids, segment_ids, attention_mask = unpack_inputs_3(inputs)
        embedding = self.bert_embedding(input_ids, segment_ids, training=training)
        # (batch_size, seq_len) -> (batch_size, 1, 1, seq_len)
        attention_mask = attention_mask[:, tf.newaxis, tf.newaxis, :]
        output, all_hidden_states, all_attention_scores = self.bert_encoder(
            embedding, attention_mask, training=training)
        pooled_output = self.bert_pooler(output)
        outputs = (output, pooled_output)
        if self.return_states:
            outputs += (all_hidden_states, )
        if self.return_attention_weights:
            outputs += (all_attention_scores, )
        return outputs

    def get_config(self):
        config = {
            'vocab_size': self.vocab_size,
            'type_vocab_size': self.type_vocab_size,
            'max_positions': self.max_positions,
            'num_layers': self.num_layers,
            'hidden_size': self.hidden_size,
            'num_attention_heads': self.num_attention_heads,
            'intermediate_size': self.intermediate_size,
            'hidden_dropout_rate': self.hidden_dropout_rate,
            'attention_dropout_rate': self.attention_dropout_rate,
            'initializer_range': self.initializer_range,
            'return_states': self.return_states,
            'return_attention_weights': self.return_attention_weights
        }
        return config



import json
import logging
import os

import tensorflow as tf

from transformers_keras.adapters import parse_pretrained_model_files
from transformers_keras.adapters.albert_adapter import AlbertAdapter

from .modeling_bert import BertEmbedding, BertEncoderLayer, BertIntermediate
from .modeling_utils import choose_activation, unpack_inputs_3


class AlbertEmbedding(tf.keras.layers.Layer):

    def __init__(self,
                 vocab_size=21128,
                 max_positions=512,
                 type_vocab_size=2,
                 embedding_size=128,
                 hidden_dropout_rate=0.2,
                 initializer_range=0.02,
                 epsilon=1e-8,
                 **kwargs):
        super().__init__(**kwargs)
        self.vocab_size = vocab_size
        self.max_positions = max_positions
        self.type_vocab_size = type_vocab_size
        self.embedding_size = embedding_size
        self.initializer_range = initializer_range

        self.layernorm = tf.keras.layers.LayerNormalization(epsilon=epsilon, name='LayerNorm')
        self.dropout = tf.keras.layers.Dropout(hidden_dropout_rate)

    def build(self, input_shape):
        self.word_embedding = self.add_weight(
            name='word_embeddings',
            shape=[self.vocab_size, self.embedding_size],
            initializer=tf.keras.initializers.TruncatedNormal(stddev=self.initializer_range))
        self.position_embedding = self.add_weight(
            name='position_embeddings',
            shape=[self.max_positions, self.embedding_size],
            initializer=tf.keras.initializers.TruncatedNormal(stddev=self.initializer_range))
        self.segment_embedding = self.add_weight(
            name='token_type_embeddings',
            shape=[self.type_vocab_size, self.embedding_size],
            initializer=tf.keras.initializers.TruncatedNormal(stddev=self.initializer_range))
        super().build(input_shape)

    def call(self, input_ids, segment_ids=None, position_ids=None, training=None):
        if segment_ids is None:
            segment_ids = tf.zeros_like(input_ids)
        if position_ids is None:
            position_ids = tf.range(0, tf.shape(input_ids)[1], dtype=input_ids.dtype)
            position_ids = tf.expand_dims(position_ids, axis=0)

        position_embeddings = tf.gather(self.position_embedding, position_ids)
        position_embeddings = tf.tile(position_embeddings, multiples=[tf.shape(input_ids)[0], 1, 1])
        token_type_embeddings = tf.gather(self.segment_embedding, segment_ids)
        token_embeddings = tf.gather(self.word_embedding, input_ids)

        embeddings = token_embeddings + token_type_embeddings + position_embeddings
        embeddings = self.layernorm(embeddings)
        embeddings = self.dropout(embeddings, training=training)
        return embeddings


class AlbertMultiHeadAttention(tf.keras.layers.Layer):

    def __init__(self,
                 hidden_size=768,
                 num_attention_heads=8,
                 hidden_dropout_rate=0.0,
                 attention_dropout_rate=0.0,
                 initializer_range=0.02,
                 epsilon=1e-8,
                 **kwargs):
        super().__init__(**kwargs)
        self.num_attention_heads = num_attention_heads
        self.hidden_size = hidden_size

        self.query_weight = tf.keras.layers.Dense(self.hidden_size, name='query')
        self.key_weight = tf.keras.layers.Dense(self.hidden_size, name='key')
        self.value_weight = tf.keras.layers.Dense(self.hidden_size, name='value')

        self.dense = tf.keras.layers.Dense(
            hidden_size,
            kernel_initializer=tf.keras.initializers.TruncatedNormal(stddev=initializer_range),
            name='dense')
        self.layer_norm = tf.keras.layers.LayerNormalization(epsilon=epsilon, name='layer_norm')
        self.attention_dropout = tf.keras.layers.Dropout(attention_dropout_rate)
        self.output_dropout = tf.keras.layers.Dropout(hidden_dropout_rate)

    def _split_heads(self, x, batch_size):
        x = tf.reshape(x, (batch_size, -1, self.num_attention_heads, self.hidden_size // self.num_attention_heads))
        return tf.transpose(x, perm=[0, 2, 1, 3])

    def _scaled_dot_product_attention(self, query, key, value, attention_mask, training=None):
        query = tf.cast(query, dtype=self.dtype)
        key = tf.cast(key, dtype=self.dtype)
        value = tf.cast(value, dtype=self.dtype)

        score = tf.matmul(query, key, transpose_b=True)
        dk = tf.cast(tf.shape(query)[-1], self.dtype)
        score = score / tf.math.sqrt(dk)
        if attention_mask is not None:
            attention_mask = tf.cast(attention_mask, dtype=self.dtype)
            score += tf.cast((1.0 - attention_mask) * -10000.0, self.dtype)
        attn_weights = tf.nn.softmax(score, axis=-1)
        attn_weights = self.attention_dropout(attn_weights, training=training)
        context = tf.matmul(attn_weights, value)
        return context, attn_weights

    def call(self, query, key, value, attention_mask, training=None):
        origin_input = query  # query == key == value

        batch_size = tf.shape(query)[0]
        query = self._split_heads(self.query_weight(query), batch_size)
        key = self._split_heads(self.key_weight(key), batch_size)
        value = self._split_heads(self.value_weight(value), batch_size)

        context, attn_weights = self._scaled_dot_product_attention(query, key, value, attention_mask, training=training)
        context = tf.transpose(context, perm=[0, 2, 1, 3])
        context = tf.reshape(context, [batch_size, -1, self.hidden_size])
        output = self.dense(context)
        output = self.output_dropout(output, training=training)
        output = self.layer_norm(output + origin_input)
        return output, attn_weights


class AlbertEncoderLayer(tf.keras.layers.Layer):

    def __init__(self,
                 hidden_size=768,
                 num_attention_heads=8,
                 intermediate_size=3072,
                 activation='gelu',
                 hidden_dropout_rate=0.0,
                 attention_dropout_rate=0.0,
                 epsilon=1e-8,
                 initializer_range=0.02,
                 **kwargs):
        super().__init__(**kwargs)

        self.attention = AlbertMultiHeadAttention(
            hidden_size=hidden_size,
            num_attention_heads=num_attention_heads,
            hidden_dropout_rate=hidden_dropout_rate,
            attention_dropout_rate=attention_dropout_rate,
            initializer_range=initializer_range,
            epsilon=epsilon,
            name='attention')

        self.ffn = tf.keras.layers.Dense(
            intermediate_size,
            kernel_initializer=tf.keras.initializers.TruncatedNormal(stddev=initializer_range),
            name='ffn')
        self.activation = choose_activation(activation)
        self.ffn_output = tf.keras.layers.Dense(
            hidden_size,
            kernel_initializer=tf.keras.initializers.TruncatedNormal(stddev=initializer_range),
            name='ffn_output')
        self.layer_norm = tf.keras.layers.LayerNormalization(epsilon=epsilon, name='layer_norm')
        self.dropout = tf.keras.layers.Dropout(hidden_dropout_rate)

    def call(self, hidden_states, attention_mask, training=None):
        attn_output, attn_weights = self.attention(
            hidden_states, hidden_states, hidden_states, attention_mask, training=training)
        outputs = self.ffn(attn_output)
        outputs = self.activation(outputs)
        outputs = self.ffn_output(outputs)
        outputs = self.dropout(outputs)
        outputs = self.layer_norm(outputs + attn_output)
        return outputs, attn_weights


class AlbertEncoderGroup(tf.keras.layers.Layer):

    def __init__(self,
                 num_layers_each_group=1,
                 hidden_size=768,
                 num_attention_heads=8,
                 intermediate_size=3072,
                 activation='gelu',
                 hidden_dropout_rate=0.2,
                 attention_dropout_rate=0.1,
                 epsilon=1e-12,
                 initializer_range=0.02,
                 **kwargs):
        super().__init__(**kwargs)

        self.encoder_layers = [
            AlbertEncoderLayer(
                hidden_size=hidden_size,
                num_attention_heads=num_attention_heads,
                intermediate_size=intermediate_size,
                activation=activation,
                hidden_dropout_rate=hidden_dropout_rate,
                attention_dropout_rate=attention_dropout_rate,
                epsilon=epsilon,
                initializer_range=initializer_range,
                name='layer_{}'.format(i)
            ) for i in range(num_layers_each_group)
        ]

    def call(self, hidden_states, attention_mask, training=None):
        group_hidden_states, group_attn_weights = [], []
        for idx, encoder in enumerate(self.encoder_layers):
            hidden_states, attn_weights = encoder(hidden_states, attention_mask)
            group_hidden_states.append(hidden_states)
            group_attn_weights.append(attn_weights)

        return hidden_states, group_hidden_states, group_attn_weights


class AlbertEncoder(tf.keras.layers.Layer):

    def __init__(self,
                 num_layers=12,
                 num_groups=1,
                 num_layers_each_group=1,
                 hidden_size=768,
                 num_attention_heads=8,
                 intermediate_size=3072,
                 activation='gelu',
                 hidden_dropout_rate=0.2,
                 attention_dropout_rate=0.1,
                 epsilon=1e-12,
                 initializer_range=0.02,
                 **kwargs):
        super(AlbertEncoder, self).__init__(**kwargs)

        self.embedding_mapping = tf.keras.layers.Dense(
            hidden_size,
            kernel_initializer=tf.keras.initializers.TruncatedNormal(stddev=initializer_range),
            name='embedding_mapping'
        )
        self.groups = [
            AlbertEncoderGroup(
                num_layers_each_group=num_layers_each_group,
                hidden_size=hidden_size,
                num_attention_heads=num_attention_heads,
                intermediate_size=intermediate_size,
                activation=activation,
                hidden_dropout_rate=hidden_dropout_rate,
                attention_dropout_rate=attention_dropout_rate,
                epsilon=epsilon,
                initializer_range=initializer_range,
                name='group_{}'.format(i),
            ) for i in range(num_groups)
        ]

        self.num_layers = num_layers
        self.num_groups = num_groups

    def call(self, hidden_states, attention_mask, training=None):
        hidden_states = self.embedding_mapping(hidden_states)

        all_hidden_states, all_attention_weights = [], []
        for i in range(self.num_layers):
            layers_per_group = self.num_layers // self.num_groups
            group_index = i // layers_per_group
            hidden_states, group_hidden_states, group_attn_weights = self.groups[group_index](
                hidden_states, attention_mask)
            all_hidden_states.extend(group_hidden_states)
            all_attention_weights.extend(group_attn_weights)

        # stack all_hidden_states to shape:
        # [batch_size, num_layers, num_attention_heads, hidden_size]
        all_hidden_states = tf.stack(all_hidden_states, axis=0)
        all_hidden_states = tf.transpose(all_hidden_states, perm=[1, 0, 2, 3])
        # stack all_attention_scores to shape:
        # [batch_size, num_layers, num_attention_heads, seqeucen_length, sequence_length]
        all_attention_weights = tf.stack(all_attention_weights, axis=0)
        all_attention_weights = tf.transpose(all_attention_weights, perm=[1, 0, 2, 3, 4])
        return hidden_states, all_hidden_states, all_attention_weights


class AlbertPooler(tf.keras.layers.Layer):

    def __init__(self, hidden_size, initializer_range=0.02, **kwargs):
        super().__init__(**kwargs)
        self.dense = tf.keras.layers.Dense(
            hidden_size,
            kernel_initializer=tf.keras.initializers.TruncatedNormal(stddev=initializer_range),
            activation='tanh',
            name='dense')

    def call(self, sequence_output):
        return self.dense(sequence_output[:, 0])


class Albert(tf.keras.Model):

    def __init__(self,
                 vocab_size=-1,
                 max_positions=512,
                 embedding_size=128,
                 type_vocab_size=2,
                 num_layers=12,
                 num_groups=1,
                 num_layers_each_group=1,
                 hidden_size=768,
                 num_attention_heads=8,
                 intermediate_size=3072,
                 activation='gelu',
                 hidden_dropout_rate=0.2,
                 attention_dropout_rate=0.1,
                 epsilon=1e-12,
                 initializer_range=0.02,
                 return_states=False,
                 return_attention_weights=False,
                 **kwargs):
        super(Albert, self).__init__(**kwargs)
        assert vocab_size > 0, "vocab_size must greater than 0."

        self.vocab_size = vocab_size
        self.type_vocab_size = type_vocab_size
        self.max_positions = max_positions
        self.embedding_size = embedding_size
        self.num_layers = num_layers
        self.num_groups = num_groups
        self.num_layers_each_group = num_layers_each_group
        self.hidden_size = hidden_size
        self.num_attention_heads = num_attention_heads
        self.intermediate_size = intermediate_size
        self.hidden_dropout_rate = hidden_dropout_rate
        self.attention_dropout_rate = attention_dropout_rate
        self.initializer_range = initializer_range
        self.initialize_range = initializer_range

        self.embedding = AlbertEmbedding(
            vocab_size=vocab_size,
            max_positions=max_positions,
            embedding_size=embedding_size,
            type_vocab_size=type_vocab_size,
            hidden_dropout_rate=hidden_dropout_rate,
            epsilon=epsilon,
            initializer_range=initializer_range,
            name='embeddings')

        self.encoder = AlbertEncoder(
            num_layers=num_layers,
            num_groups=num_groups,
            num_layers_each_group=num_layers_each_group,
            hidden_size=hidden_size,
            num_attention_heads=num_attention_heads,
            intermediate_size=intermediate_size,
            activation=activation,
            hidden_dropout_rate=hidden_dropout_rate,
            attention_dropout_rate=attention_dropout_rate,
            epsilon=epsilon,
            initializer_range=initializer_range,
            name='encoder')

        self.pooler = AlbertPooler(hidden_size=hidden_size, initializer_range=initializer_range, name='pooler')

        self.return_states = return_states
        self.return_attention_weights = return_attention_weights

    @tf.function(input_signature=[
        {
            'input_ids': tf.TensorSpec(shape=(None, None), dtype=tf.int32, name='input_ids'),
            'segment_ids': tf.TensorSpec(shape=(None, None), dtype=tf.int32, name='segment_ids'),
            'attention_mask': tf.TensorSpec(shape=(None, None), dtype=tf.int32, name='attention_mask')
        }
    ])
    def serving(self, inputs):
        input_ids, segment_ids, attention_mask = inputs['input_ids'], inputs['segment_ids'], inputs['attention_mask']
        outputs = self(input_ids, segment_ids, attention_mask)
        outputs = list(outputs)
        results = {
            'sequence_output': outputs.pop(0),
            'pooled_output': outputs.pop(0),
        }
        if self.return_states:
            results['hidden_states'] = outputs.pop(0)
        if self.return_attention_weights:
            results['attention_weights'] = outputs.pop(0)
        return results

    def call(self, input_ids, segment_ids=None, attention_mask=None, training=None):
        input_ids, segment_ids, attention_mask = unpack_inputs_3([input_ids, segment_ids, attention_mask])
        attention_mask = attention_mask[:, tf.newaxis, tf.newaxis, :]
        embed = self.embedding(input_ids, segment_ids)
        sequence_outputs, all_hidden_states, all_attn_weights = self.encoder(embed, attention_mask)
        # take [CLS]
        pooled_output = self.pooler(sequence_outputs)
        outputs = (sequence_outputs, pooled_output)
        if self.return_states:
            outputs += (all_hidden_states, )
        if self.return_attention_weights:
            outputs += (all_attn_weights, )
        return outputs

    def dummy_inputs(self):
        input_ids = tf.constant([0] * 128, dtype=tf.int64, shape=(1, 128))
        segment_ids = tf.constant([0] * 128, dtype=tf.int64, shape=(1, 128))
        attn_mask = tf.constant([1] * 128, dtype=tf.int64, shape=(1, 128))
        return input_ids, segment_ids, attn_mask

    @classmethod
    def from_pretrained(cls, pretrained_model_dir, adapter=None, verbose=True, **kwargs):
        config_file, ckpt, vocab_file = parse_pretrained_model_files(pretrained_model_dir)
        if not adapter:
            adapter = AlbertAdapter(**kwargs)
        model_config = adapter.adapte_config(config_file, **kwargs)
        model_config['return_states'] = kwargs.get('return_states', False)
        model_config['return_attention_weights'] = kwargs.get('return_attention_weights', False)
        model = cls(**model_config)
        input_ids, segment_ids, attn_mask = model.dummy_inputs()
        model(input_ids, segment_ids, attn_mask)
        adapter.adapte_weights(model, model_config, ckpt, **kwargs)
        return model

    def get_config(self):
        config = {
            'vocab_size': self.vocab_size,
            'type_vocab_size': self.type_vocab_size,
            'max_positions': self.max_positions,
            'num_layers': self.num_layers,
            'hidden_size': self.hidden_size,
            'num_attention_heads': self.num_attention_heads,
            'intermediate_size': self.intermediate_size,
            'hidden_dropout_rate': self.hidden_dropout_rate,
            'attention_dropout_rate': self.attention_dropout_rate,
            'initializer_range': self.initializer_range,
            'return_states': self.return_states,
            'return_attention_weights': self.return_attention_weights
        }
        return config



import json
import logging
import os

import numpy as np
import tensorflow as tf

from .abstract_adapter import AbstractAdapter, zip_weights


class AlbertAdapter(AbstractAdapter):

    def __init__(self, skip_embedding_mapping_in=False, **kwargs):
        super().__init__(**kwargs)
        self.skip_embedding_mapping_in = skip_embedding_mapping_in

    def adapte_config(self, config_file, **kwargs):
        with open(config_file, mode='rt', encoding='utf8') as fin:
            config = json.load(fin)

        model_config = {
            'vocab_size': config['vocab_size'],
            'max_positions': config['max_position_embeddings'],
            'embedding_size': config['embedding_size'],
            'type_vocab_size': config['type_vocab_size'],
            'num_layers': config['num_hidden_layers'],
            'num_groups': config['num_hidden_groups'],
            'num_layers_each_group': config['inner_group_num'],
            'hidden_size': config['hidden_size'],
            'num_attention_heads': config['num_attention_heads'],
            'intermediate_size': config['intermediate_size'],
            'activation': config['hidden_act'],
            'hidden_dropout_rate': config['hidden_dropout_prob'],
            'attention_dropout_rate': config['attention_probs_dropout_prob'],
            'initializer_range': config['initializer_range'],
        }
        return model_config

    def adapte_weights(self, model, config, ckpt, **kwargs):
        # mapping weight names
        weights_mapping = {}
        ckpt_prefix = kwargs.get('ckpt_prefix', 'bert')
        for cw in [x[0] for x in tf.train.list_variables(ckpt)]:
            if any(x in cw for x in ['embeddings/', 'pooler/']):
                weights_mapping[model.name + cw.lstrip(ckpt_prefix) + ':0'] = cw
                continue
            if 'embedding_hidden_mapping_in/kernel' in cw:
                k = '{}/encoder/embedding_mapping/kernel:0'.format(model.name)
                v = '{}/encoder/embedding_hidden_mapping_in/kernel'.format(ckpt_prefix)
                weights_mapping[k] = v
            if 'embedding_hidden_mapping_in/bias' in cw:
                k = '{}/encoder/embedding_mapping/bias:0'.format(model.name)
                v = '{}/encoder/embedding_hidden_mapping_in/bias'.format(ckpt_prefix)
                weights_mapping[k] = v

        mapping = self._mapping_weight_names(
            config['num_groups'], config['num_layers_each_group'], model_name=model.name)
        weights_mapping.update(mapping)

        # filter
        if self.skip_token_embedding:
            self._skip_weight(weights_mapping, model.name + '/embeddings/word_embeddings:0')
        if self.skip_position_embedding:
            self._skip_weight(weights_mapping, model.name + '/embeddings/position_embeddings:0')
        if self.skip_segment_embedding:
            self._skip_weight(weights_mapping, model.name + '/embeddings/token_type_embeddings:0')
        if self.skip_embedding_layernorm:
            self._skip_weight(weights_mapping, model.name + '/embeddings/LayerNorm/gamma:0')
            self._skip_weight(weights_mapping, model.name + '/embeddings/LayerNorm/beta:0')
        if self.skip_pooler:
            self._skip_weight(weights_mapping, model.name + '/pooler/dense/kernel:0')
            self._skip_weight(weights_mapping, model.name + '/pooler/dense/bias:0')
        if self.skip_embedding_mapping_in:
            self._skip_weight(weights_mapping, model.name + '/encoder/embedding_mapping/kernel:0')
            self._skip_weight(weights_mapping, model.name + '/encoder/embedding_mapping/bias:0')

        # zip weights and its' values
        zipped_weights = zip_weights(
            model,
            ckpt,
            weights_mapping,
            verbose=kwargs.get('verbose', True))
        # set values to weights
        tf.keras.backend.batch_set_value(zipped_weights)

        self_weights = {w.name: w.numpy() for w in model.trainable_weights}
        for k, v in self_weights.items():
            ckpt_key = weights_mapping.get(k, None)
            if not ckpt_key:
                continue
            ckpt_value = tf.train.load_variable(ckpt, ckpt_key)
            if ckpt_value is None:
                logging.warning('ckpt value is None of key: %s', ckpt_key)
            assert np.allclose(v, ckpt_value)

    def _mapping_weight_names(self, num_groups, num_layers_each_group, model_name='albert', ckpt_prefix='bert'):
        logging.info('Using model_name: %s', model_name)
        mapping = {}

        # encoder
        for group in range(num_groups):
            for layer in range(num_layers_each_group):
                k_prefix = '{}/encoder/group_{}/layer_{}/'.format(model_name, group, layer)
                v_prefix = '{}/encoder/transformer/group_{}/inner_group_{}/'.format(ckpt_prefix, group, layer)
                # attention
                for n in ['query', 'key', 'value']:
                    for x in ['kernel', 'bias']:
                        k = k_prefix + 'attention/{}/{}:0'.format(n, x)
                        v = v_prefix + 'attention_1/self/{}/{}'.format(n, x)
                        mapping[k] = v

                # attention dense
                for n in ['kernel', 'bias']:
                    k = k_prefix + 'attention/dense/{}:0'.format(n)
                    v = v_prefix + 'attention_1/output/dense/{}'.format(n)
                    mapping[k] = v

                for n in ['gamma', 'beta']:
                    # attention layer norm
                    k = k_prefix + 'attention/layer_norm/{}:0'.format(n)
                    v = v_prefix + 'LayerNorm/{}'.format(n)
                    mapping[k] = v
                    # albert encoder layer norm
                    k = k_prefix + 'layer_norm/{}:0'.format(n)
                    v = v_prefix + 'LayerNorm_1/{}'.format(n)
                    mapping[k] = v

                for n in ['kernel', 'bias']:
                    # intermediate
                    k = k_prefix + 'ffn/{}:0'.format(n)
                    v = v_prefix + 'ffn_1/intermediate/dense/{}'.format(n)
                    mapping[k] = v
                    # dense
                    k = k_prefix + 'ffn_output/{}:0'.format(n)
                    v = v_prefix + 'ffn_1/intermediate/output/dense/{}'.format(n)
                    mapping[k] = v

        return mapping

    def _skip_weight(self, mapping, name):
        mapping.pop(name)
        logging.info('Skip load weight: %s', name)



import numpy as np
import tensorflow as tf
from transformers_keras.modeling_albert import (Albert, AlbertEmbedding,
                                                AlbertEncoder,
                                                AlbertEncoderGroup,
                                                AlbertEncoderLayer)


class ModelingAlbertTest(tf.test.TestCase):

    def testAlbertEmbedding(self):
        embedding = AlbertEmbedding(vocab_size=100, embedding_size=128)
        inputs = (
            tf.constant([[0, 2, 3, 4, 5, 1]]),  # input_ids
            tf.constant([[0, 0, 0, 1, 1, 1]]),  # token_type_ids
            # tf.constant([[1, 1, 1, 1, 1, 1]]),  # attention_mask
        )
        outputs = embedding(inputs[0], inputs[1], training=True)
        self.assertAllEqual([1, 6, 128], outputs.shape)

    def testAlbertEncoderLayer(self):
        encoder = AlbertEncoderLayer()
        hidden_states = tf.random.uniform(shape=(2, 16, 768))  # hidden states
        mask = tf.constant([[0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1],
                            [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1]],
                           shape=(2, 16), dtype=tf.float32)  # mask
        attention_mask = mask[:, tf.newaxis, tf.newaxis, :]
        outputs, attn_weights = encoder(hidden_states, attention_mask)
        self.assertAllEqual([2, 16, 768], outputs.shape)
        self.assertAllEqual([2, 8, 16, 16], attn_weights.shape)

    def testAlbertEncoderGroup(self):

        hidden_states = tf.random.uniform(shape=(2, 16, 768))  # hidden states
        mask = tf.constant([[0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1],
                            [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1]],
                           shape=(2, 16), dtype=tf.float32)  # mask
        mask = mask[:, tf.newaxis, tf.newaxis, :]

        def _run_albert_encoder_group(num_layers_each_group):
            group = AlbertEncoderGroup(num_layers_each_group=num_layers_each_group)
            outputs, group_states, group_attn_weights = group(hidden_states, mask)
            self.assertAllEqual([2, 16, 768], outputs.shape)

            self.assertEqual(num_layers_each_group, len(group_states))
            for state in group_states:
                self.assertAllEqual([2, 16, 768], state.shape)

            self.assertEqual(num_layers_each_group, len(group_attn_weights))
            for attention in group_attn_weights:
                self.assertAllEqual([2, 8, 16, 16], attention.shape)

        for layer in [1, 2, 3, 4]:
            _run_albert_encoder_group(layer)

    def testAlbertEncoder(self):
        hidden_states = tf.random.uniform(shape=(2, 16, 768))  # hidden states
        mask = tf.constant([[0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1],
                            [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1]],
                           shape=(2, 16), dtype=tf.float32)  # mask
        mask = mask[:, tf.newaxis, tf.newaxis, :]

        NUM_LAYERS = 4
        NUM_GROUPS = 1
        encoder = AlbertEncoder(num_layers=NUM_LAYERS, num_groups=NUM_GROUPS, num_layers_each_group=1, hidden_size=768)

        outputs, all_states, all_attn_weights = encoder(hidden_states, mask)
        self.assertAllEqual([2, 16, 768], outputs.shape)

        self.assertAllEqual([2, NUM_LAYERS * NUM_GROUPS, 16, 768], all_states.shape)
        self.assertAllEqual([2, NUM_LAYERS * NUM_GROUPS, 8, 16, 16], all_attn_weights.shape)

    def _build_albert_inputs(self):
        input_ids = tf.constant(
            [1, 2, 3, 4, 5, 6, 7, 5, 3, 2, 3, 4, 1, 2, 3, 1, 2, 3, 4, 5, 6, 6, 6, 7, 7, 8, 0, 0, 0, 0, 0, 0],
            shape=(2, 16),
            dtype=np.int32)  # input_ids
        token_type_ids = np.array(
            [[0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1],
             [0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1]], dtype=np.int64)  # token_type_ids,
        input_mask = tf.constant(
            [[1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0],
             [1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0]], dtype=np.float32)  # input_mask
        return input_ids, token_type_ids, input_mask

    def _check_albert_outputs(self, return_states=False, return_attention_weights=False):
        NUM_LAYERS, NUM_GROUPS, NUM_LAYERS_EACH_GROUP = 4, 1, 1
        model = Albert(
            vocab_size=100, hidden_size=768,
            num_layers=NUM_LAYERS, num_groups=NUM_GROUPS, num_layers_each_group=NUM_LAYERS_EACH_GROUP,
            return_states=return_states,
            return_attention_weights=return_attention_weights)
        input_ids, segment_ids, attn_mask = self._build_albert_inputs()
        outputs = model(input_ids, segment_ids, attn_mask)
        sequence_outputs, pooled_outputs = outputs[0], outputs[1]
        self.assertAllEqual([2, 16, 768], sequence_outputs.shape)
        self.assertAllEqual([2, 768], pooled_outputs.shape)

        all_states, all_attn_weights = None, None
        if return_states and return_attention_weights:
            self.assertEqual(4, len(outputs))
            all_states, all_attn_weights = outputs[2], outputs[3]
        elif return_states and not return_attention_weights:
            self.assertEqual(3, len(outputs))
            all_states = outputs[2]
        elif not return_states and return_attention_weights:
            self.assertEqual(3, len(outputs))
            all_attn_weights = outputs[2]
        else:
            self.assertEqual(2, len(outputs))

        if all_states is not None:
            self.assertAllEqual([2, NUM_LAYERS, 16, 768], all_states.shape)

        if all_attn_weights is not None:
            self.assertAllEqual([2, NUM_LAYERS, 8, 16, 16], all_attn_weights.shape)

    def testAlbert(self):
        self._check_albert_outputs(return_states=True, return_attention_weights=True)
        self._check_albert_outputs(return_states=True, return_attention_weights=False)
        self._check_albert_outputs(return_states=False, return_attention_weights=True)
        self._check_albert_outputs(return_states=False, return_attention_weights=False)

    def test_build_model(self):
        model = Albert(vocab_size=21128)
        input_ids, segment_ids, attn_mask = model.dummy_inputs()
        model(input_ids, segment_ids, attn_mask)
        model.summary()

        for v in model.trainable_weights:
            print(v.name)

    def test_albert_config(self):
        model = Albert(vocab_size=21128)
        print(model.get_config())

    def test_export_saved_model(self):
        model = Albert(vocab_size=21128, num_layers=12, num_attention_heads=8,
                       return_states=True, return_attention_weights=True)
        input_ids, segment_ids, input_mask = model.dummy_inputs()
        model(input_ids=input_ids, segment_ids=segment_ids, attention_mask=input_mask)
        model.summary()
        model.save('models/albert/export/1', include_optimizer=False, signatures=model.serving)

    def test_load_saved_model(self):
        loaded = tf.saved_model.load('models/albert/export/1')
        model = loaded.signatures['serving_default']
        print(model.structured_input_signature)
        print(model.structured_outputs)


if __name__ == "__main__":
    tf.test.main()
